@@ -27,10 +27,12 @@ def generate_temp_dir(path):
     """Temporary directory generation
 
     Generates a temporary file with a random name at the input path.
+    
     Parameters
     ----------
     path : str
         The path at which the temporary directory will be created.
+    
     Returns
     -------
     str
@@ -53,12 +55,50 @@ def generate_temp_dir(path):
     return full_path
 
 
-def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len=20):
+def check_bt2_index(ref):
+    """
+    Checks for the existence of a bowtie2 index based on the reference
+    file name.
+
+    Parameters
+    ----------
+    ref : str
+        Path to the reference genome.
+
+    Returns
+    -------
+    index : str
+        The bowtie2 index basename.
+    """
+    index = os.path.splitext(ref)[0]
+    try:
+        index = glob.glob(index + "*rev.1.bt2")[0]
+        index = index.split(".rev.1.bt2")[0]
+    except IndexError:
+        logger.error(
+            "Reference index is missing, please build the bowtie2 "
+            "index first."
+        )
+        sys.exit(1)
+    return index
+
+
+def iterative_align(
+    fq_in,
+    tmp_dir,
+    ref,
+    n_cpu,
+    sam_out,
+    minimap2=False,
+    min_len=20,
+    min_qual=30,
+):
     """Iterative alignment
 
     Aligns reads iteratively reads of fq_in with bowtie2 or minimap2. Reads are
-    truncated to the 40 first nucleotides and unmapped reads are extended by 20
+    truncated to the 20 first nucleotides and unmapped reads are extended by 20
     nucleotides and realigned on each iteration.
+
     Parameters
     ----------
     fq_in : str
@@ -75,6 +115,8 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
         If True, use minimap2 instead of bowtie2 for the alignment.
     min_len : int
         The initial length of the fragments to align.
+    min_qual : int
+        Minimum mapping quality required to keep Hi-C pairs.
     """
     # set with the name of the unaligned reads :
     remaining_reads = set()
@@ -101,16 +143,9 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
         uncomp_path = fq_in
 
     # throw error if index does not exist
-    index = os.path.splitext(ref)[0]
+    index = ""
     if not minimap2:
-        try:
-            index = glob.glob(index + "*rev.1.bt2")[0]
-            index = index.rstrip(".rev.1.bt2")
-        except IndexError:
-            logger.error(
-                "Reference index is missing, please build the bowtie2 " "index first."
-            )
-            sys.exit(1)
+        index = check_bt2_index(ref)
     # Counting reads
     with ct.read_compressed(uncomp_path) as inf:
         for _ in inf:
@@ -132,21 +167,23 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
             "min_len must be shorter than the reads. Either decrease it or do not use iterative mapping."
         )
         sys.exit(1)
-    logger.info("{0} reads to parse".format(total_reads))
+    logger.info("{0} reads to parse".format(int(total_reads)))
 
     # iterative alignment per se
     while n <= size:
-        logger.info("\n" + "-" * 10 + "\nn = {0}".format(n))
+        logger.info(
+            "Truncating unaligned reads to {0}bp and mapping again.".format(
+                int(n)
+            )
+        )
         iter_out += [os.path.join(tmp_dir, "trunc_{0}.sam".format(str(n)))]
         # Generate a temporary input fastq file with the n first nucleotids
         # of the reads.
-        logger.info("Generating truncated reads")
         truncated_reads = truncate_reads(
             tmp_dir, uncomp_path, remaining_reads, n, min_len
         )
 
         # Align the truncated reads on reference genome
-        logger.info("Aligning reads")
         temp_alignment = "{0}/temp_alignment.sam".format(tmp_dir)
         map_args = {
             "fa": ref,
@@ -156,7 +193,9 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
             "idx": index,
         }
         if minimap2:
-            cmd = "minimap2 -x sr -a -t {threads} {fa} {fq} > {sam}".format(**map_args)
+            cmd = "minimap2 -x sr -a -t {threads} {fa} {fq} > {sam}".format(
+                **map_args
+            )
         else:
             cmd = (
                 "bowtie2 -x {idx} -p {threads} --rdg 500,3 --rfg 500,3"
@@ -167,18 +206,21 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
         # filter the reads: the reads whose truncated end was aligned are written
         # to the output file.
         # The reads whose truncated end was not aligned are kept for the next round.
-        logger.info("Reporting aligned reads")
-        remaining_reads = filter_samfile(temp_alignment, iter_out[-1])
+        remaining_reads = filter_samfile(
+            temp_alignment, iter_out[-1], min_qual
+        )
 
         n += 20
 
     # one last round without trimming
-    logger.info("\n" + "-" * 10 + "\nn = {0}".format(size))
-    logger.info("Generating truncated reads")
+    logger.info(
+        "Trying to map unaligned reads at full length ({0}bp).".format(
+            int(size)
+        )
+    )
     truncated_reads = truncate_reads(
         tmp_dir, uncomp_path, remaining_reads, size, min_len
     )
-    logger.info("Aligning reads")
     if minimap2:
         cmd = "minimap2 -x sr -a -t {1} {0} {3} > {2}".format(
             ref, n_cpu, temp_alignment, truncated_reads
@@ -189,9 +231,8 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
             "--very-sensitive -S {2} {3}"
         ).format(index, n_cpu, temp_alignment, truncated_reads)
     sp.call(cmd, shell=True)
-    logger.info("Reporting aligned reads")
     iter_out += [os.path.join(tmp_dir, "trunc_{0}.sam".format(str(n)))]
-    remaining_reads = filter_samfile(temp_alignment, iter_out[-1])
+    remaining_reads = filter_samfile(temp_alignment, iter_out[-1], min_qual)
 
     # Report unaligned reads as well
     iter_out += [os.path.join(tmp_dir, "unaligned.sam")]
@@ -208,7 +249,7 @@ def iterative_align(fq_in, tmp_dir, ref, n_cpu, sam_out, minimap2=False, min_len
     ps.merge("-O", "SAM", "-@", str(n_cpu), sam_out, *iter_out)
     logger.info(
         "{0} reads aligned / {1} total reads.".format(
-            total_reads - len(remaining_reads), total_reads
+            int(total_reads - len(remaining_reads)), int(total_reads)
         )
     )
 
@@ -245,20 +286,25 @@ def truncate_reads(tmp_dir, infile, unaligned_set, n, min_len):
     return outfile
 
 
-def filter_samfile(temp_alignment, filtered_out):
+def filter_samfile(temp_alignment, filtered_out, min_qual=30):
     """Filter alignment SAM files
 
     Reads all the reads in the input SAM alignment file.
     Write reads to the output file if they are aligned with a good
     quality, otherwise add their name in a set to stage them for the next round
     of alignment.
+
     Parameters
     ----------
     temp_alignment : str
         Path to the input temporary alignment.
     outfile : str
         Path to the output filtered temporary alignment.
+    min_qual : int
+        Minimum mapping quality required to keep a Hi-C pair.
+    
     Returns
+    -------
     set:
         Contains the names reads that did not align.
     """
@@ -270,7 +316,7 @@ def filter_samfile(temp_alignment, filtered_out):
     temp_sam = ps.AlignmentFile(temp_alignment, "r")
     outf = ps.AlignmentFile(filtered_out, "w", template=temp_sam)
     for r in temp_sam:
-        if r.flag in [0, 16] and r.mapping_quality >= 30:
+        if r.flag in [0, 16] and r.mapping_quality >= min_qual:
             outf.write(r)
         else:
             unaligned.add(r.query_name)
